package function.builtin.predicate;

import function.*;
import sexpression.*;

public class GREATERP extends LispFunction {

    private ArgumentValidator argumentValidator;

    public GREATERP() {
        this.argumentValidator = new ArgumentValidator(">");
        this.argumentValidator.setMinimumNumberOfArguments(1);
        this.argumentValidator.setEveryArgumentExpectedType(LispNumber.class);
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        return callTailRecursive(argumentList);
    }

    private SExpression callTailRecursive(Cons argumentList) {
        Cons remainingArguments = (Cons) argumentList.getRest();

        if (remainingArguments.isNull())
            return Symbol.T;

        SExpression firstArgument = argumentList.getFirst();
        SExpression secondArgument = remainingArguments.getFirst();
        LispNumber number1 = (LispNumber) firstArgument;
        LispNumber number2 = (LispNumber) secondArgument;

        if (!isFirstGreater(number1, number2))
            return Nil.getInstance();

        return callTailRecursive(remainingArguments);
    }

    private boolean isFirstGreater(LispNumber number1, LispNumber number2) {
        return number1.getValue().compareTo(number2.getValue()) > 0;
    }

}
